@@ -1,4 +1,5 @@
 --------------------------------------------------------------------------------
+{-# LANGUAGE LambdaCase        #-}
 {-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE RecordWildCards   #-}
 module Patat.Presentation.Display
@@ -11,19 +12,23 @@ module Patat.Presentation.Display
 
 --------------------------------------------------------------------------------
 import           Control.Monad                        (guard)
+import           Control.Monad.Identity               (runIdentity)
+import           Control.Monad.Writer                 (Writer, execWriter, tell)
 import qualified Data.Aeson.Extended                  as A
 import           Data.Char.WCWidth.Extended           (wcstrwidth)
-import           Data.Data.Extended                   (grecQ)
+import           Data.Foldable                        (for_)
+import qualified Data.HashMap.Strict                  as HMS
 import qualified Data.List                            as L
 import           Data.Maybe                           (fromMaybe, maybeToList)
 import qualified Data.Sequence.Extended               as Seq
 import qualified Data.Text                            as T
-import qualified Patat.Presentation.Comments          as Comments
 import           Patat.Presentation.Display.CodeBlock
 import           Patat.Presentation.Display.Internal
 import           Patat.Presentation.Display.Table
 import           Patat.Presentation.Internal
 import           Patat.Presentation.Settings
+import qualified Patat.Presentation.SpeakerNotes      as SpeakerNotes
+import           Patat.Presentation.Syntax
 import           Patat.PrettyPrint                    ((<$$>), (<+>))
 import qualified Patat.PrettyPrint                    as PP
 import           Patat.Size
@@ -31,7 +36,6 @@ import           Patat.Theme                          (Theme (..))
 import qualified Patat.Theme                          as Theme
 import           Prelude
 import qualified Text.Pandoc.Extended                 as Pandoc
-import qualified Text.Pandoc.Writers.Shared           as Pandoc
 
 
 --------------------------------------------------------------------------------
@@ -56,16 +60,23 @@ displayWithBorders (Size rows columns) pres@Presentation {..} f =
     PP.hardline
   where
     -- Get terminal width/title
-    (sidx, _) = pActiveFragment
-    settings  = activeSettings pres
-    ds        = DisplaySettings
+    settings     = activeSettings pres
+    (sidx, _)    = pActiveFragment
+    ds           = DisplaySettings
         { dsSize          = canvasSize
-        , dsWrap          = fromMaybe NoWrap $ psWrap settings
         , dsMargins       = margins settings
+        , dsWrap          = fromMaybe NoWrap $ psWrap settings
+        , dsTabStop       = maybe 4 A.unFlexibleNum $ psTabStop settings
         , dsTheme         = fromMaybe Theme.defaultTheme (psTheme settings)
         , dsSyntaxMap     = pSyntaxMap
+        , dsResolve       = \var -> fromMaybe [] $ HMS.lookup var pVars
+        , dsRevealState   = revealState
         }
 
+    revealState = case activeFragment pres of
+        Just (ActiveContent _ _ c) -> c
+        _                          -> mempty
+
     -- Compute title.
     breadcrumbs = fromMaybe [] $ Seq.safeIndex pBreadcrumbs sidx
     plainTitle  = PP.toString $ prettyInlines ds pTitle
@@ -102,26 +113,25 @@ displayPresentation :: Size -> Presentation -> Display
 displayPresentation size pres@Presentation {..} =
      case activeFragment pres of
         Nothing -> DisplayDoc $ displayWithBorders size pres mempty
-        Just (ActiveContent fragment)
+        Just (ActiveContent fragment _ _)
                 | Just _ <- psImages pSettings
                 , Just image <- onlyImage fragment ->
             DisplayImage $ T.unpack image
-        Just (ActiveContent fragment) -> DisplayDoc $
+        Just (ActiveContent fragment _ _) -> DisplayDoc $
             displayWithBorders size pres $ \theme ->
-                prettyFragment theme fragment
+                prettyMargins theme fragment
         Just (ActiveTitle block) -> DisplayDoc $
             displayWithBorders size pres $ \ds ->
                 let auto = Margins {mTop = Auto, mRight = Auto, mLeft = Auto} in
-                prettyFragment ds {dsMargins = auto} $ Fragment [block]
-
+                prettyMargins ds {dsMargins = auto} [block]
   where
     -- Check if the fragment consists of "just a single image".  Discard
     -- headers.
-    onlyImage (Fragment (Pandoc.Header{} : bs)) = onlyImage (Fragment bs)
-    onlyImage (Fragment bs) = case bs of
-        [Pandoc.Figure _ _ bs']                      -> onlyImage (Fragment bs')
-        [Pandoc.Para [Pandoc.Image _ _ (target, _)]] -> Just target
-        _                                            -> Nothing
+    onlyImage (Header{} : bs) = onlyImage bs
+    onlyImage bs = case bs of
+        [Figure _ bs']                 -> onlyImage bs'
+        [Para [Image _ _ (target, _)]] -> Just target
+        _                              -> Nothing
 
 
 --------------------------------------------------------------------------------
@@ -144,23 +154,21 @@ dumpPresentation pres@Presentation {..} =
     dumpSlide :: Int -> [PP.Doc]
     dumpSlide i = do
         slide <- maybeToList $ getSlide i pres
-        dumpComment slide <> L.intercalate ["{fragment}"]
+        dumpSpeakerNotes slide <> L.intercalate ["{fragment}"]
             [ dumpFragment (i, j)
             | j <- [0 .. numFragments slide - 1]
             ]
 
-    dumpComment :: Slide -> [PP.Doc]
-    dumpComment slide = do
-        guard (Comments.cSpeakerNotes comment /= mempty)
+    dumpSpeakerNotes :: Slide -> [PP.Doc]
+    dumpSpeakerNotes slide = do
+        guard (slideSpeakerNotes slide /= mempty)
         pure $ PP.text $ "{speakerNotes: " <>
-            Comments.speakerNotesToText (Comments.cSpeakerNotes comment) <> "}"
-      where
-        comment = slideComment slide
+            SpeakerNotes.toText (slideSpeakerNotes slide) <> "}"
 
     dumpFragment :: Index -> [PP.Doc]
     dumpFragment idx =
         case displayPresentation (getSize idx) pres {pActiveFragment = idx} of
-            DisplayDoc doc        -> [doc]
+            DisplayDoc   doc      -> [doc]
             DisplayImage filepath -> [PP.string $ "{image: " ++ filepath ++ "}"]
 
     getSize :: Index -> Size
@@ -172,31 +180,68 @@ dumpPresentation pres@Presentation {..} =
 
 
 --------------------------------------------------------------------------------
-prettyFragment :: DisplaySettings -> Fragment -> PP.Doc
-prettyFragment ds (Fragment blocks) = vertical $
-    PP.vcat (map (horizontal . prettyBlock ds) blocks) <>
+-- | Renders the given blocks, adding margins based on the settings and wrapping
+-- based on width.
+prettyMargins :: DisplaySettings -> [Block] -> PP.Doc
+prettyMargins ds blocks = vertical $
+    map horizontal blocks ++
     case prettyReferences ds blocks of
-        []   -> mempty
-        refs -> PP.hardline <> PP.vcat (map horizontal refs)
+        []   -> []
+        refs ->
+            let doc0        = PP.vcat refs
+                size@(r, _) = PP.dimensions doc0 in
+            [(horizontalIndent size $ horizontalWrap doc0, r)]
   where
     Size rows columns = dsSize ds
     Margins {..} = dsMargins ds
 
-    vertical doc0 =
-        mconcat (replicate top PP.hardline) <> doc0
+    -- For every block, calculate the size based on its last fragment.
+    blockSize block =
+        let revealState = blocksRevealLastStep [block] in
+        PP.dimensions $ deindent $ horizontalWrap $
+            prettyBlock ds {dsRevealState = revealState} block
+
+    -- Vertically align some blocks by adding spaces in front of it.
+    -- We also take in the number of rows for every block so we don't
+    -- need to recompute it.
+    vertical :: [(PP.Doc, Int)] -> PP.Doc
+    vertical docs0 = mconcat (replicate top PP.hardline) <> doc
       where
         top = case mTop of
-            Auto -> let (r, _) = PP.dimensions doc0 in (rows - r) `div` 2
+            Auto      -> (rows - actual) `div` 2
             NotAuto x -> x
 
-    horizontal = horizontalIndent . horizontalWrap
+        docs1  = [verticalPad r d | (d, r) <- docs0]
+        actual = sum $ L.intersperse 1 $ map snd docs1
+        doc    = PP.vcat $ map fst docs1
 
-    horizontalIndent doc0 = PP.indent indentation indentation doc1
+    -- Vertically pad a doc by adding lines below it.
+    -- Return the actual size as well as the padded doc.
+    verticalPad :: Int -> PP.Doc -> (PP.Doc, Int)
+    verticalPad desired doc0
+        | actual >= rows = (doc0, actual)
+        | otherwise      = (doc0 <> padding, desired)
       where
-        doc1 = case (mLeft, mRight) of
-            (Auto, Auto) -> PP.deindent doc0
-            _            -> doc0
-        (_, dcols) = PP.dimensions doc1
+        (actual, _) = PP.dimensions doc0
+        padding     = mconcat $ replicate (desired - actual) PP.hardline
+
+    -- Render and horizontally align a block.  Also returns the desired rows.
+    horizontal :: Block -> (PP.Doc, Int)
+    horizontal b@(Reveal ConcatWrapper reveal) =
+        -- Horizontally aligning a fragment with a ConcatWrapper is a special
+        -- case, as we want to horizontal align all the things inside
+        -- individually.
+        let (fblocks, _) = unzip $ map horizontal $
+                revealToBlocks (dsRevealState ds) ConcatWrapper reveal in
+        (PP.vcat fblocks, fst (blockSize b))
+    horizontal block =
+        let size@(r, _) = blockSize block in
+        (horizontalIndent size $ horizontalWrap $ prettyBlock ds block, r)
+
+    horizontalIndent :: (Int, Int) -> PP.Doc -> PP.Doc
+    horizontalIndent (_, dcols) doc0 = PP.indent indentation indentation doc1
+      where
+        doc1 = deindent doc0
         left = case mLeft of
             NotAuto x -> x
             Auto      -> case mRight of
@@ -204,6 +249,13 @@ prettyFragment ds (Fragment blocks) = vertical $
                 Auto      -> (columns - dcols) `div` 2
         indentation = PP.Indentation left mempty
 
+    -- Strip leading spaces to horizontally align code blocks etc.
+    deindent doc0 = case (mLeft, mRight) of
+        (Auto, Auto) -> PP.deindent doc0
+        _            -> doc0
+
+    -- Rearranges lines to fit into the wrap settings.
+    horizontalWrap :: PP.Doc -> PP.Doc
     horizontalWrap doc0 = case dsWrap ds of
         NoWrap     -> doc0
         AutoWrap   -> PP.wrapAt (Just $ columns - right - left) doc0
@@ -218,21 +270,21 @@ prettyFragment ds (Fragment blocks) = vertical $
 
 
 --------------------------------------------------------------------------------
-prettyBlock :: DisplaySettings -> Pandoc.Block -> PP.Doc
+prettyBlock :: DisplaySettings -> Block -> PP.Doc
 
-prettyBlock ds (Pandoc.Plain inlines) = prettyInlines ds inlines
+prettyBlock ds (Plain inlines) = prettyInlines ds inlines
 
-prettyBlock ds (Pandoc.Para inlines) =
+prettyBlock ds (Para inlines) =
     prettyInlines ds inlines <> PP.hardline
 
-prettyBlock ds (Pandoc.Header i _ inlines) =
+prettyBlock ds (Header i _ inlines) =
     themed ds themeHeader (PP.string (replicate i '#') <+> prettyInlines ds inlines) <>
     PP.hardline
 
-prettyBlock ds (Pandoc.CodeBlock (_, classes, _) txt) =
+prettyBlock ds (CodeBlock (_, classes, _) txt) =
     prettyCodeBlock ds classes txt
 
-prettyBlock ds (Pandoc.BulletList bss) = PP.vcat
+prettyBlock ds (BulletList bss) = PP.vcat
     [ PP.indent
         (PP.Indentation 2 $ themed ds themeBulletList prefix)
         (PP.Indentation 4 mempty)
@@ -253,7 +305,7 @@ prettyBlock ds (Pandoc.BulletList bss) = PP.vcat
         }
     ds'    = ds {dsTheme = theme'}
 
-prettyBlock ds (Pandoc.OrderedList _ bss) = PP.vcat
+prettyBlock ds (OrderedList _ bss) = PP.vcat
     [ PP.indent
         (PP.Indentation 0 $ themed ds themeOrderedList $ PP.string prefix)
         (PP.Indentation 4 mempty)
@@ -267,15 +319,15 @@ prettyBlock ds (Pandoc.OrderedList _ bss) = PP.vcat
         | i <- [1 .. length bss]
         ]
 
-prettyBlock _ds (Pandoc.RawBlock _ t) = PP.text t <> PP.hardline
+prettyBlock _ds (RawBlock _ t) = PP.text t <> PP.hardline
 
-prettyBlock _ds Pandoc.HorizontalRule = "---"
+prettyBlock _ds HorizontalRule = "---"
 
-prettyBlock ds (Pandoc.BlockQuote bs) =
+prettyBlock ds (BlockQuote bs) =
     let quote = PP.Indentation 0 (themed ds themeBlockQuote "> ") in
     PP.indent quote quote (themed ds themeBlockQuote $ prettyBlocks ds bs)
 
-prettyBlock ds (Pandoc.DefinitionList terms) =
+prettyBlock ds (DefinitionList terms) =
     PP.vcat $ map prettyDefinition terms
   where
     prettyDefinition (term, definitions) =
@@ -284,134 +336,157 @@ prettyBlock ds (Pandoc.DefinitionList terms) =
         [ PP.indent
             (PP.Indentation 0 (themed ds themeDefinitionList ":   "))
             (PP.Indentation 4 mempty) $
-            prettyBlocks ds (Pandoc.plainToPara definition)
+            prettyBlocks ds (plainToPara definition)
         | definition <- definitions
         ]
 
-prettyBlock ds (Pandoc.Table _ caption specs thead tbodies tfoot) =
+    plainToPara :: [Block] -> [Block]
+    plainToPara = map $ \case
+        Plain inlines -> Para inlines
+        block         -> block
+
+
+prettyBlock ds (Table caption aligns headers rows) =
     PP.wrapAt Nothing $
-    prettyTable ds Table
-        { tCaption = prettyInlines ds caption'
-        , tAligns  = map align aligns
-        , tHeaders = map (prettyBlocks ds) headers
-        , tRows    = map (map (prettyBlocks ds)) rows
+    prettyTableDisplay ds TableDisplay
+        { tdCaption = prettyInlines ds caption
+        , tdAligns  = map align aligns
+        , tdHeaders = map (prettyBlocks ds) headers
+        , tdRows    = map (map (prettyBlocks ds)) rows
         }
   where
-    (caption', aligns, _, headers, rows) = Pandoc.toLegacyTable
-        caption specs thead tbodies tfoot
-
     align Pandoc.AlignLeft    = PP.AlignLeft
     align Pandoc.AlignCenter  = PP.AlignCenter
     align Pandoc.AlignDefault = PP.AlignLeft
     align Pandoc.AlignRight   = PP.AlignRight
 
-prettyBlock ds (Pandoc.Div _attrs blocks) = prettyBlocks ds blocks
+prettyBlock ds (Div _attrs blocks) = prettyBlocks ds blocks
 
-prettyBlock ds (Pandoc.LineBlock inliness) =
+prettyBlock ds (LineBlock inliness) =
     let ind = PP.Indentation 0 (themed ds themeLineBlock "| ") in
     PP.wrapAt Nothing $
     PP.indent ind ind $
     PP.vcat $
     map (prettyInlines ds) inliness
 
-prettyBlock ds (Pandoc.Figure _attr _caption blocks) =
-    prettyBlocks ds blocks
+prettyBlock ds (Figure _attr blocks) = prettyBlocks ds blocks
+
+prettyBlock ds (Reveal w fragment) = prettyBlocks ds $
+    revealToBlocks (dsRevealState ds) w fragment
+
+prettyBlock ds (VarBlock var) = prettyBlocks ds $ dsResolve ds var
+
+prettyBlock _ (SpeakerNote _) = mempty
+prettyBlock _ (Config _) = mempty
 
 
 --------------------------------------------------------------------------------
-prettyBlocks :: DisplaySettings -> [Pandoc.Block] -> PP.Doc
+prettyBlocks :: DisplaySettings -> [Block] -> PP.Doc
 prettyBlocks ds = PP.vcat . map (prettyBlock ds)
 
 
 --------------------------------------------------------------------------------
-prettyInline :: DisplaySettings -> Pandoc.Inline -> PP.Doc
+prettyInline :: DisplaySettings -> Inline -> PP.Doc
 
-prettyInline _ds Pandoc.Space = PP.space
+prettyInline _ds Space = PP.space
 
-prettyInline _ds (Pandoc.Str str) = PP.text str
+prettyInline _ds (Str str) = PP.text str
 
-prettyInline ds (Pandoc.Emph inlines) =
+prettyInline ds (Emph inlines) =
     themed ds themeEmph $
     prettyInlines ds inlines
 
-prettyInline ds (Pandoc.Strong inlines) =
+prettyInline ds (Strong inlines) =
     themed ds themeStrong $
     prettyInlines ds inlines
 
-prettyInline ds (Pandoc.Underline inlines) =
+prettyInline ds (Underline inlines) =
     themed ds themeUnderline $
     prettyInlines ds inlines
 
-prettyInline ds (Pandoc.Code _ txt) =
+prettyInline ds (Code _ txt) =
     themed ds themeCode $
     PP.text (" " <> txt <> " ")
 
-prettyInline ds link@(Pandoc.Link _attrs text (target, _title))
-    | isReferenceLink link =
+prettyInline ds link@(Link _attrs _text (target, _title))
+    | Just (text, _, _) <- toReferenceLink link =
         "[" <> themed ds themeLinkText (prettyInlines ds text) <> "]"
     | otherwise =
         "<" <> themed ds themeLinkTarget (PP.text target) <> ">"
 
-prettyInline _ds Pandoc.SoftBreak = PP.softline
+prettyInline _ds SoftBreak = PP.softline
 
-prettyInline _ds Pandoc.LineBreak = PP.hardline
+prettyInline _ds LineBreak = PP.hardline
 
-prettyInline ds (Pandoc.Strikeout t) =
+prettyInline ds (Strikeout t) =
     "~~" <> themed ds themeStrikeout (prettyInlines ds t) <> "~~"
 
-prettyInline ds (Pandoc.Quoted Pandoc.SingleQuote t) =
+prettyInline ds (Quoted Pandoc.SingleQuote t) =
     "'" <> themed ds themeQuoted (prettyInlines ds t) <> "'"
-prettyInline ds (Pandoc.Quoted Pandoc.DoubleQuote t) =
+prettyInline ds (Quoted Pandoc.DoubleQuote t) =
     "'" <> themed ds themeQuoted (prettyInlines ds t) <> "'"
 
-prettyInline ds (Pandoc.Math _ t) =
+prettyInline ds (Math _ t) =
     themed ds themeMath (PP.text t)
 
-prettyInline ds (Pandoc.Image _attrs text (target, _title)) =
+prettyInline ds (Image _attrs text (target, _title)) =
     "![" <> themed ds themeImageText (prettyInlines ds text) <> "](" <>
     themed ds themeImageTarget (PP.text target) <> ")"
 
-prettyInline _ (Pandoc.RawInline _ t) = PP.text t
+prettyInline _ (RawInline _ t) = PP.text t
 
 -- These elements aren't really supported.
-prettyInline ds  (Pandoc.Cite      _ t) = prettyInlines ds t
-prettyInline ds  (Pandoc.Span      _ t) = prettyInlines ds t
-prettyInline ds  (Pandoc.Note        t) = prettyBlocks  ds t
-prettyInline ds  (Pandoc.Superscript t) = prettyInlines ds t
-prettyInline ds  (Pandoc.Subscript   t) = prettyInlines ds t
-prettyInline ds  (Pandoc.SmallCaps   t) = prettyInlines ds t
+prettyInline ds  (Cite      _ t) = prettyInlines ds t
+prettyInline ds  (Span      _ t) = prettyInlines ds t
+prettyInline _   (Note        _) = mempty  -- TODO: support notes?
+prettyInline ds  (Superscript t) = prettyInlines ds t
+prettyInline ds  (Subscript   t) = prettyInlines ds t
+prettyInline ds  (SmallCaps   t) = prettyInlines ds t
 -- prettyInline unsupported = PP.ondullred $ PP.string $ show unsupported
 
 
 --------------------------------------------------------------------------------
-prettyInlines :: DisplaySettings -> [Pandoc.Inline] -> PP.Doc
+prettyInlines :: DisplaySettings -> [Inline] -> PP.Doc
 prettyInlines ds = mconcat . map (prettyInline ds)
 
 
 --------------------------------------------------------------------------------
-prettyReferences :: DisplaySettings -> [Pandoc.Block] -> [PP.Doc]
-prettyReferences ds =
-    map prettyReference . getReferences
-  where
-    getReferences :: [Pandoc.Block] -> [Pandoc.Inline]
-    getReferences = filter isReferenceLink . grecQ
+type Reference = ([Inline], T.Text, T.Text)
 
-    prettyReference :: Pandoc.Inline -> PP.Doc
-    prettyReference (Pandoc.Link _attrs text (target, title)) =
+
+--------------------------------------------------------------------------------
+prettyReferences :: DisplaySettings -> [Block] -> [PP.Doc]
+prettyReferences ds =
+    map prettyReference . execWriter . dftBlocks (pure . pure) tellReference
+  where
+    tellReference :: Inline -> Writer [Reference] [Inline]
+    tellReference inline = do
+        for_ (toReferenceLink inline) (tell . pure)
+        pure [inline]
+
+    prettyReference :: Reference -> PP.Doc
+    prettyReference (text, target, title) =
         "[" <>
         themed ds themeLinkText
-            (prettyInlines ds $ Pandoc.newlineToSpace text) <>
+            (prettyInlines ds $ newlineToSpace text) <>
         "](" <>
         themed ds themeLinkTarget (PP.text target) <>
         (if T.null title
             then mempty
             else PP.space <> "\"" <> PP.text title <> "\"")
         <> ")"
-    prettyReference _ = mempty
+
+    newlineToSpace :: [Inline] -> [Inline]
+    newlineToSpace = runIdentity . dftInlines (pure . pure) work
+      where
+        work x = pure $ case x of
+            SoftBreak -> [Space]
+            LineBreak -> [Space]
+            _         -> [x]
 
 
 --------------------------------------------------------------------------------
-isReferenceLink :: Pandoc.Inline -> Bool
-isReferenceLink (Pandoc.Link _attrs text (target, _)) =
-    [Pandoc.Str target] /= text
-isReferenceLink _ = False
+toReferenceLink :: Inline -> Maybe Reference
+toReferenceLink (Link _attrs text (target, title))
+    | [Str target] /= text = Just (text, target, title)
+toReferenceLink _ = Nothing
