@@ -8,6 +8,7 @@ module Patat.Presentation.Display.Internal
 --------------------------------------------------------------------------------
 import           Patat.Presentation.Internal (Margins)
 import           Patat.Presentation.Settings (Wrap)
+import           Patat.Presentation.Syntax   (Block, RevealState, Var)
 import qualified Patat.PrettyPrint           as PP
 import           Patat.Size                  (Size)
 import qualified Patat.Theme                 as Theme
@@ -16,11 +17,14 @@ import qualified Skylighting                 as Skylighting
 
 --------------------------------------------------------------------------------
 data DisplaySettings = DisplaySettings
-    { dsSize      :: !Size
-    , dsWrap      :: !Wrap
-    , dsMargins   :: !Margins
-    , dsTheme     :: !Theme.Theme
-    , dsSyntaxMap :: !Skylighting.SyntaxMap
+    { dsSize        :: !Size
+    , dsWrap        :: !Wrap
+    , dsTabStop     :: !Int
+    , dsMargins     :: !Margins
+    , dsTheme       :: !Theme.Theme
+    , dsSyntaxMap   :: !Skylighting.SyntaxMap
+    , dsResolve     :: !(Var -> [Block])
+    , dsRevealState :: !RevealState
     }
 
 
