@@ -2,37 +2,17 @@
 {-# LANGUAGE LambdaCase #-}
 module Text.Pandoc.Extended
     ( module Text.Pandoc
-
-    , plainToPara
-    , newlineToSpace
-
     , readPlainText
     ) where
 
 
 --------------------------------------------------------------------------------
-import           Data.Char          (isSpace)
-import           Data.Data.Extended (grecT)
-import qualified Data.Text          as T
+import           Data.Char   (isSpace)
+import qualified Data.Text   as T
 import           Prelude
 import           Text.Pandoc
 
 
---------------------------------------------------------------------------------
-plainToPara :: [Block] -> [Block]
-plainToPara = map $ \case
-    Plain inlines -> Para inlines
-    block         -> block
-
-
---------------------------------------------------------------------------------
-newlineToSpace :: [Inline] -> [Inline]
-newlineToSpace = grecT $ \case
-    SoftBreak -> Space
-    LineBreak -> Space
-    inline    -> inline
-
-
 --------------------------------------------------------------------------------
 -- | A plain-text reader.  Always returns empty metadata.
 readPlainText :: T.Text -> Pandoc
